package Exercise5 
  import HeatLib;
  annotation (Coordsys(
      extent=[0, 0; 442, 394], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.45, 
      y=0.01, 
      width=0.44, 
      height=0.65, 
      library=1, 
      autolayout=1));
  HeatLib Heat;
  testheat1 solution1;
  testheat2 solution2;
end Exercise5;


model testheat1 
  package SI = Modelica.SIunits;
  parameter SI.Length L=1.0 "Length of rod";
  parameter SI.Area A=0.0004 "Area of rod";
  parameter SI.Density rho=7.5*1000 "Density of rod material";
  parameter SI.ThermalConductivity lambda=74 
    "Thermal conductivity of material";
  parameter SI.SpecificHeatCapacity c=450 "spec. heat capacity of material";
  parameter SI.Temp_C T0=20 "initial temperature";
protected 
  parameter Real n=3 "number of elements";
  parameter SI.Length Lelem=L/n "length of a heat capacity element";
  parameter SI.Mass mElem=A*Lelem*rho "mass of a heat capacity element";
  annotation (Coordsys(
      extent=[-120, -100; 120, 100], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.08, 
      y=0.02, 
      width=0.91, 
      height=0.83));
public 
  HeatLib.HeatResistance heatR1(
    A=A, 
    L=Lelem/2, 
    lambda=lambda) annotation (extent=[-40, 20; -20, 40]);
  HeatLib.HeatResistance heatR2(
    A=A, 
    L=Lelem, 
    lambda=lambda) annotation (extent=[0, 20; 20, 40]);
  HeatLib.HeatResistance heatR3(
    A=A, 
    L=Lelem, 
    lambda=lambda) annotation (extent=[40, 20; 60, 40]);
  HeatLib.HeatCapacitance heatC1(
    m=mElem, 
    c=c, 
    T0=T0) annotation (extent=[-20, 0; 0, 20]);
  HeatLib.HeatCapacitance heatC2(
    m=mElem, 
    c=c, 
    T0=T0) annotation (extent=[20, 0; 40, 20]);
  HeatLib.HeatCapacitance heatC3(
    m=mElem, 
    c=c, 
    T0=T0) annotation (extent=[60, 0; 80, 20]);
  Modelica.Blocks.Sources.Constant const(k={200}) annotation (extent=[-100, 20
        ; -80, 40]);
  HeatLib.TempSource_C Tsource annotation (extent=[-70, 20; -50, 40]);
  HeatLib.HeatResistance heatR4(
    A=A, 
    L=Lelem/2, 
    lambda=lambda) annotation (extent=[80, 20; 100, 40]);
equation 
  connect(heatR1.surface_b, heatR2.surface_a) annotation (points=[-20, 30; 0, 
        30]);
  connect(heatR1.surface_b, heatC1.surface_a) annotation (points=[-20, 30; -10
        , 30; -10, 10]);
  connect(heatR2.surface_b, heatR3.surface_a) annotation (points=[20, 30; 40, 
        30]);
  connect(heatR2.surface_b, heatC2.surface_a) annotation (points=[20, 30; 30, 
        30; 30, 10]);
  connect(heatR3.surface_b, heatC3.surface_a) annotation (points=[60, 30; 70, 
        30; 70, 10]);
  connect(Tsource.surface_b, heatR1.surface_a) annotation (points=[-50, 30; -
        40, 30]);
  connect(const.outPort, Tsource.inPort) annotation (points=[-79, 30; -71, 30]
    );
  connect(heatR4.surface_a, heatR3.surface_b) annotation (points=[80, 30; 60, 
        30]);
end testheat1;


model testheat2 
  package SIunits = Modelica.SIunits;
  parameter SIunits.Length L=1 "lenght of rod";
  parameter SIunits.Area A=0.0004 "area of rod";
  parameter SIunits.Density rho=7.5*1000 "density of rod material";
  parameter SIunits.ThermalConductivity lambda=74 
    "thermal conductivity of material";
  parameter SIunits.SpecificHeatCapacity c=450 
    "specifc heat capacity of material";
  parameter SIunits.Temp_C T0=20 "initial temperature";
  
  HeatLib.TempSource_C Tsource annotation (extent=[-28, 20; -8, 40]);
  annotation (
    Coordsys(
      extent=[-100, -100; 100, 100], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.23, 
      y=0, 
      width=0.77, 
      height=0.79), 
    Documentation(info="Compare rod3.C[1:3].surface_a.T with 
        rod15.C[{3,8,12}].surface_a.T
"));
  HeatLib.HeatedRod rod3(
    n=3, 
    L=L, 
    A=A, 
    rho=rho, 
    lambda=lambda, 
    c=c, 
    T0=T0) annotation (extent=[20, 40; 60, 80]);
  Modelica.Blocks.Sources.Constant const(k={200}) annotation (extent=[-60, 20
        ; -40, 40]);
  HeatLib.HeatedRod rod15(
    n=15, 
    L=L, 
    A=A, 
    rho=rho, 
    lambda=lambda, 
    c=c, 
    T0=T0) annotation (extent=[20, -20; 60, 20]);
equation 
  connect(const.outPort, Tsource.inPort) annotation (points=[-39, 30; -29, 30]
    );
  connect(Tsource.surface_b, rod3.surface_a) annotation (points=[-8, 30; 4, 30
        ; 4, 60; 20, 60], style(color=41));
  connect(Tsource.surface_b, rod15.surface_a) annotation (points=[-8, 30; 4, 
        30; 4, 0; 20, 0], style(color=41));
end testheat2;

