package HeatLib "1-dimensional heat flow" 
  extends Modelica.Icons.Library;
  package SIunits = Modelica.SIunits;
  
  package Interfaces "Connectors and partial models for 1D heat flow" 
    extends Modelica.Icons.Library;
    
    connector Surface_a "1D Heat flow connector (filled icon)" 
      SIunits.Temperature T "Absolute temperature";
      flow SIunits.HeatFlux q "Heat flowing into element";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.23, 
          y=0.02, 
          width=0.56, 
          height=0.68), 
        Icon(Rectangle(extent=[-100, -100; 100, 100], style(color=41, 
                fillColor=41))), 
        Diagram(Rectangle(extent=[-100, -100; 100, 100], style(color=41, 
                fillColor=41)), Text(
            extent=[-100, -120; 100, -220], 
            string="%name", 
            style(color=0))));
    end Surface_a;
    connector Surface_b "1D Heat flow connector (non-filled icon)" 
      SIunits.Temperature T "Absolute temperature";
      flow SIunits.HeatFlux q "Heat flowing into element";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.23, 
          y=0.02, 
          width=0.56, 
          height=0.68), 
        Icon(Rectangle(extent=[-100, -100; 100, 100], style(color=41, 
                fillColor=7))), 
        Diagram(Rectangle(extent=[-100, -100; 100, 100], style(color=41, 
                fillColor=7)), Text(
            extent=[-100, -120; 100, -220], 
            string="%name", 
            style(color=0))));
    end Surface_b;
    
    annotation (Coordsys(
        extent=[0, 0; 188, 115], 
        grid=[1, 1], 
        component=[20, 20]), Window(
        x=0.05, 
        y=0.04, 
        width=0.19, 
        height=0.21, 
        library=1, 
        autolayout=1));
  end Interfaces;
  model TempSource_C "Celsius temperature source" 
    Interfaces.Surface_b surface_b annotation (extent=[90, -10; 110, 10]);
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.21, 
        y=0.11, 
        width=0.6, 
        height=0.57), 
      Icon(
        Rectangle(extent=[-40, 100; 20, -60], style(color=0)), 
        Line(points=[-100, 0; -40, 0]), 
        Rectangle(extent=[20, 10; 90, -10], style(
            color=1, 
            fillColor=1, 
            fillPattern=1)), 
        Rectangle(extent=[-40, 88; -12, 78], style(
            color=8, 
            fillColor=9, 
            fillPattern=1)), 
        Rectangle(extent=[-40, 58; -12, 48], style(
            color=8, 
            fillColor=9, 
            fillPattern=1)), 
        Rectangle(extent=[-40, 30; -12, 20], style(
            color=8, 
            fillColor=9, 
            fillPattern=1)), 
        Rectangle(extent=[-40, 0; -12, -10], style(
            color=8, 
            fillColor=9, 
            fillPattern=1)), 
        Rectangle(extent=[-40, -30; -12, -40], style(
            color=8, 
            fillColor=9, 
            fillPattern=1)), 
        Text(
          extent=[-100, 60; -100, 30], 
          string="T", 
          style(color=0)), 
        Text(extent=[0, -76; 0, -136], string="%name"), 
        Text(
          extent=[-100, -30; -100, -60], 
          string="[degC]", 
          style(color=0))));
    Modelica.Blocks.Interfaces.InPort inPort(final n=1) annotation (extent=[-
          120, -10; -100, 10]);
  equation 
    surface_b.T = inPort.signal[1] - Modelica.Constants.T_zero;
  end TempSource_C;
  model HeatResistance "Ideal heat flow without storage of energy" 
    parameter SIunits.Area A(min=0) "area of heat resistance";
    parameter SIunits.Length L(min=Modelica.Constants.eps) 
      "length of heat resistance";
    parameter SIunits.ThermalConductivity lambda(min=0) 
      "thermal conductivity of material";
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.13, 
        y=0.02, 
        width=0.63, 
        height=0.7), 
      Icon(Rectangle(extent=[-90, 20; 90, -20], style(
            gradient=2, 
            fillColor=1, 
            fillPattern=1)), Text(extent=[0, 91; 0, 31], string="%name")), 
      Diagram(Polygon(points=[-81, 10; -78, 10; 40, 10; 40, 30; 80, 0; 40, -30
              ; 40, -10; -80, -10; -80, 10; -81, 10], style(color=41, fillColor
              =41))));
    Interfaces.Surface_a surface_a annotation (extent=[-110, -10; -90, 10]);
    Interfaces.Surface_b surface_b annotation (extent=[90, -10; 110, 10]);
  equation 
    surface_a.q = lambda*A/L*(surface_a.T - surface_b.T);
    surface_b.q = -surface_a.q;
  end HeatResistance;
  model HeatCapacitance "Ideal heat storage in a block without heat flow" 
    parameter SIunits.Mass m(min=0) "mass of block";
    parameter SIunits.SpecificHeatCapacity c(min=0) 
      "specifc heat capacity of block";
    parameter SIunits.Temp_C T0=20 "initial temperature of block";
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.13, 
        y=0.06, 
        width=0.63, 
        height=0.63), 
      Icon(Rectangle(extent=[-100, -10; 100, -80], style(
            color=1, 
            gradient=3, 
            fillColor=1, 
            fillPattern=1)), Text(extent=[0, -89; 0, -149], string="%name")));
    Interfaces.Surface_a surface_a(T(final start=T0 - Modelica.Constants.
            T_zero)) annotation (extent=[-10, -10; 10, 10]);
  equation 
    surface_a.q = c*m*der(surface_a.T);
  end HeatCapacitance;
  model HeatedRod 
    parameter Integer n=3 "number of discretization elements";
    parameter SIunits.Length L=1 "lenght of rod";
    parameter SIunits.Area A=0.0004 "area of rod";
    parameter SIunits.Density rho=7.5*1000 "density of rod material";
    parameter SIunits.ThermalConductivity lambda=74 
      "thermal conductivity of material";
    parameter SIunits.SpecificHeatCapacity c=450 
      "specifc heat capacity of material";
    parameter SIunits.Temp_C T0=20 "initial temperature";
    SIunits.Temp_C T[n] "Temperature at the heat capacity elements";
  protected 
    parameter SIunits.Length Lelem=L/n "length of a heat capacity element";
    parameter SIunits.Mass mElem=A*Lelem*rho "mass of a heat capacity element"
      ;
  public 
    Interfaces.Surface_a surface_a annotation (extent=[-110, -10; -90, 10]);
    Interfaces.Surface_b surface_b annotation (extent=[90, -10; 110, 10]);
  protected 
    HeatResistance R0(
      A=A, 
      L=Lelem/2, 
      lambda=lambda) annotation (extent=[-50, -10; -30, 10]);
    HeatResistance Rn(
      A=A, 
      L=Lelem/2, 
      lambda=lambda) annotation (extent=[30, -10; 50, 10]);
    HeatResistance R[n - 1](
      A=A, 
      L=Lelem, 
      lambda=lambda) annotation (extent=[-10, -10; 10, 10]);
    HeatCapacitance C[n](
      m=mElem, 
      c=c, 
      T0=T0) annotation (extent=[-10, -40; 10, -20]);
    annotation (
      Icon(
        Rectangle(extent=[-88, 20; 90, -20], style(
            gradient=2, 
            fillColor=1, 
            fillPattern=1)), 
        Text(extent=[-98, 98; 100, 36], string="%name"), 
        Line(points=[-60, 20; -60, -18], style(color=0)), 
        Line(points=[20, 20; 20, -18], style(color=0)), 
        Line(points=[60, 18; 60, -20], style(color=0)), 
        Text(
          extent=[-100, -30; 100, -70], 
          string="n=%n", 
          style(color=0)), 
        Line(points=[-20, 18; -20, -20], style(color=0))), 
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.13, 
        y=0.03, 
        width=0.66, 
        height=0.67));
  equation 
    connect(surface_a, R0.surface_a) annotation (points=[-100, 0; -50, 0]);
    connect(surface_b, Rn.surface_b) annotation (points=[100, 0; 50, 0]);
    connect(R0.surface_b, C[1].surface_a);
    connect(Rn.surface_a, C[n].surface_a);
    
    for i in 1:n - 1 loop
      connect(C[i].surface_a, R[i].surface_a);
      connect(R[i].surface_b, C[i + 1].surface_a);
    end for;
    
    // determine temperature
    for i in 1:n loop
      T[i] = C[i].surface_a.T + Modelica.Constants.T_zero;
    end for;
  end HeatedRod;
  annotation (Coordsys(
      extent=[0, 0; 442, 394], 
      grid=[1, 1], 
      component=[20, 20]), Window(
      x=0.45, 
      y=0.01, 
      width=0.44, 
      height=0.65, 
      library=1, 
      autolayout=1));
end HeatLib;
